﻿; OpenWithLite.ahk[UTF-8] (2025/03/08 23:00)
; for AutoHotKey v2.0.19 or later
;
; Open In Firefox Browser/Open In Chrome Browser から中継起動されるランチャ
; ※似た名前のAddOn/拡張機能が存在しますが起動パスと起動オプションを任意に設定できるタイプである必要あり
;
; Firefox系用 Open in Chrome™ Browser
;   https://addons.mozilla.org/ja/firefox/addon/open-in-chrome-browser
; Chrome系用 Open in Firefox™ Browser
;   https://chromewebstore.google.com/detail/open-in-firefox-browser/lmeddoobegbaiopohmpmmobpnpjifpii
;
; Usage:
;    OpenWithLite.exe "from" URL
;       "from"
;          起動元のブラウザ名文字列
;          (iniファイルに定義したものと一致する必要あり、選択肢に自分自身を入れないため)
;          (AddOn/拡張機能のオプションで起動オプションとして送信元文字列を定義出来る必要があります）
;       URL
;          Open In XXX Browserにて起動オプションに展開されるURL(&Expanded-URLs;または&Separated-URLs;)
;          ※内部的にURLの前に " -URL "が付加されます
;
; OpenWithLite.ini に選択肢となるブラウザ一覧の定義が必要です
;


#NoTrayIcon
#SingleInstance Force

;//ToolTipはスクリーンを対象とする
CoordMode("ToolTip", "Screen")
/*
;//MouseGetPosはスクリーンを対象とする
CoordMode("Mouse", "Screen")
;//Menuの座標はスクリーンを対象とする
CoordMode("Menu", "Screen")

MouseGetPos( &posX, &posY, &posWin, &posCtrl )
*/

;//パラメータは２つ以上渡されるのが正常動作
if( A_Args.Length < 2 ){
	MsgBox( "ERROR: ArgsCount = " A_Args.Length "`r`nUsage:`r`n  OpenWithLite.exe [FROM] [URL]" )
	ExitApp
}

;//第一パラメータfromが "" で囲ってあるにも関わらず何故か内側のスペースで分離されて渡されるので再連結する
strFrom := ""
idx := 1
Loop (A_Args.Length - 1)
{
	strFrom := strFrom StrReplace(A_Args[idx],'"','') " "
	idx++
}
strFrom := Trim(strFrom)

;//最終パラメータがURL
strURL := A_Args[A_Args.Length]

;strToolTips := "[" strFrom "]" strURL
;ToolTip( strToolTips, 0, 0, 1 )

;iniファイルから定義件数を読み込む
IniFile := "OpenWithLite.ini"
IniSect := "APP"
count := IniRead(IniFile, IniSect, "MAX", 0)
if( count = 0 ){
	MsgBox( "ERROR: " IniFile "ファイルに定義がありません" )
	ExitApp()
}
strEditor     := IniRead(IniFile, IniSect, "EDIT", "C:\Windows\notepad.exe")
strBackColor  := IniRead(IniFile, IniSect, "COLOR", "FFFFFF")
iDispMine     := Number(IniRead(IniFile, IniSect, "DISPMINE", 0))
iEnableMine   := Number(IniRead(IniFile, IniSect, "ENABLEMINE", 0))
iDispNotExist := Number(IniRead(IniFile, IniSect, "DISPNOTEXIST", 0))
iEnableNewWin := Number(IniRead(IniFile, IniSect, "ENABLENEWWIN", 0))

;//ポップアップメニューを作成
MyMenu := Menu()

aryName := Array()
aryPath := Array()
aryIcon := Array()
aryOpts := Array()
idx := 1
pos := 1
Loop count
{
	strName := ""
	strPath := ""
	strIcon := ""
	strOpts := ""
	strTemp := ""
	iMine := 0
	iNotExist := 0

	;名前の読み込み(NAME1～) 両端の "" は除去
	strName := StrReplace(IniRead(IniFile, IniSect, "NAME" String(idx), ""),'"','')
	if( strName = strFrom ){
		if( iDispMine = 0  ){
			;MsgBox("strName[" strName "] is mine. Skip!")
			idx := idx + 1
			Continue
		}else{
			iMine := 1
		}
	}

	;フルパスの読み込み(PATH1～) 両端の "" はそのまま使う
	strPath := IniRead(IniFile, IniSect, "PATH" String(idx), "")
	if( ! FileExist(strPath) ){
		if( iDispNotExist = 0 ){
			;MsgBox("strPath[" strPath "] is not found. Skip!")
			idx := idx + 1
			Continue
		}else{
			iNotExist := 1
		}
	}

	;アイコンの読み込み(ICON1～) 両端の "" はそのまま使い定義が無ければパス=EXEを指定
	strIcon := IniRead(IniFile, IniSect, "ICON" String(idx), strPath)

	;起動オプションの読み込み(OPTION1～) %AAA% の"%"は環境変数として扱うので必ず前後セットで使う
	strOpts := IniRead(IniFile, IniSect, "OPTION" String(idx), "")
	if( InStr(strOpts, "%") > 0 ){
		aryTemp := StrSplit(strOpts, "%")
		strOpts := ""
		pos := 1
		Loop aryTemp.Length
		{
			strTemp := aryTemp[pos]
			;// %AAA% で区切られている時は偶数インデックスが環境変数部
			if( Mod(pos,2) = 0 ){
				strOpts := strOpts EnvGet(strTemp)
			}else{
				strOpts := strOpts strTemp
			}
			pos++
		}
	}

	aryName.push( strName )
	aryPath.push( strPath )
	aryIcon.push( strIcon )
	aryOpts.push( strOpts )

	MyMenu.Add(strName, MenuHandler)
	try{
		MyMenu.SetIcon(strName, strIcon)
	}
	if( (iMine and (not iEnableMine)) or iNotExist ){
		MyMenu.Disable(strName)
	}

	idx++
}

;メニュー選択イベントハンドラ
MenuHandler(Item, *) {
	idx := 1
	strPrefix := " -URL "
	if( iEnableNewWin and GetKeyState("Shift") ){
		strPrefix := " -new-window "
	}
	Loop aryName.Length
	{
		if( Item = aryName[idx] ){
			try{
				;//何故か新規に起動した場合にOpenWithLiteが終了すると開いたブラウザまで終了してしまう。解せぬ。
				;Run( aryPath[idx] ' ' aryOpts[idx] strPrefix strURL )
				;//親プロセスから確実に切り離してcmd.exeからstartで起動する
				Run( A_ComSpec ' /c start "" "' aryPath[idx] '" ' aryOpts[idx] strPrefix strURL ,,"Hide" )
			}catch Error as err{
				MsgBox( "Caught Error: " type(err) " [" err.Message "]" )
			}
			Break
		}
		idx++
	}
}

;オプション(ini)編集
MenuHandlerOpt(Item, *) {
	;MsgBox( A_ScriptDir "\" IniFile )
	Run( A_ComSpec ' /c start "" "' strEditor '" "' A_ScriptDir "\" IniFile '"',,"Hide" )
}


;//セパレータを追加
MyMenu.Add()
;//iniファイル編集のメニューを追加
MyMenu.Add("OpenWithLite オプション", MenuHandlerOpt)
;//メニュー背景色
MyMenu.SetColor(strBackColor)
;//メニューを表示
MyMenu.Show()


